@@ -48,115 +48,140 @@ GtkWidget *helpitem;
 GtkWidget *menubar;
 GtkWidget *toolbar;
 
-static void restart_game ()
+static void
+restart_game ()
 {
-  if (waiting_for_mouse_up()) return;
+  if (waiting_for_mouse_up ())
+    return;
   /* Treat a restart as part of the same game. Eventually either
-   * the player will win or loose and then it gets counted. */
+   * the player will win or lose and then it gets counted. */
   game_in_progress = FALSE;
   option_list_set_sensitive ();
   new_game (NULL, &seed);
 };
 
-static void new_game_action ()
+static void
+new_game_action ()
 {
-  if (waiting_for_mouse_up()) return;
+  if (waiting_for_mouse_up ())
+    return;
   new_game (NULL, NULL);
 };
 
-void undo_callback ()
+
+static void
+new_recent_game_action (GtkAction * action, gpointer user_data)
 {
-  if (waiting_for_mouse_up()) return;
+  if (user_data == NULL)
+    new_game (NULL, NULL);
+  else {
+    new_game ((gchar *) user_data, NULL);
+    g_free (user_data);
+  }
+};
+
+void
+undo_callback ()
+{
+  if (waiting_for_mouse_up ())
+    return;
   scm_c_eval_string ("(undo)");
-  refresh_screen();
+  refresh_screen ();
 }
 
-void redo_callback ()
+void
+redo_callback ()
 {
-  if (waiting_for_mouse_up()) return;
+  if (waiting_for_mouse_up ())
+    return;
   scm_c_eval_string ("(redo)");
-  refresh_screen();
+  refresh_screen ();
   /* We need this now that you can undo a losing move. */
   end_of_game_test ();
 }
 
-static void help_about_callback (void)
+static void
+help_about_callback (void)
 {
   const gchar *authors[] = {
-	  _("Main game:"),
-	  "Jonathan Blandford (jrb@redhat.com)",
-	  "Felix Bellaby (felix@pooh.u-net.com)",
-	  "Rosanna Yuen (zana@webwynk.net)",
-	  "Callum McKenzie (callum@physics.otago.ac.nz)",
-	  "",
-	  _("Card games:"),
-	  "Jonathan Blandford (jrb@redhat.com)",
-	  "W. Borgert (debacle@debian.org)",
-	  "Robert Brady (rwb197@ecs.soton.ac.uk)",
-	  "Nick Lamb (njl195@zepler.org.uk)",
-	  "Changwoo Ryu (cwryu@adam.kaist.ac.kr)",
-	  "Matthew Wilcox (matthew@wil.cx)",
-	  "Rosanna Yuen (zana@webwynk.net)",
-	  NULL
+    _("Main game:"),
+    "Jonathan Blandford (jrb@redhat.com)",
+    "Felix Bellaby (felix@pooh.u-net.com)",
+    "Rosanna Yuen (zana@webwynk.net)",
+    "Callum McKenzie (callum@physics.otago.ac.nz)",
+    "",
+    _("Card games:"),
+    "Jonathan Blandford (jrb@redhat.com)",
+    "W. Borgert (debacle@debian.org)",
+    "Robert Brady (rwb197@ecs.soton.ac.uk)",
+    "Nick Lamb (njl195@zepler.org.uk)",
+    "Changwoo Ryu (cwryu@adam.kaist.ac.kr)",
+    "Matthew Wilcox (matthew@wil.cx)",
+    "Rosanna Yuen (zana@webwynk.net)",
+    NULL
   };
 
   const gchar *documenters[] = {
-	  "Rosanna Yuen (zana@webwynk.net)",
-	  NULL
+    "Rosanna Yuen (zana@webwynk.net)",
+    NULL
   };
+  gchar *license = games_get_license (_("AisleRiot"));
 
   gtk_show_about_dialog (GTK_WINDOW (app),
 			 "name", _("AisleRiot"),
 			 "version", VERSION,
-			 "comments", _("AisleRiot provides a rule-based solitaire\n"
-				       "card engine that allows many different\n"
-				       "games to be played."),
-			 "copyright", "Copyright \xc2\xa9 1998-2006 Jonathan Blandford",
-			 "license", "GPL 2+",
-			 "authors", authors,
-			 "documenters", documenters,
-			 "translator_credits", _("translator-credits"),
-			 "logo-icon-name", "gnome-aisleriot",
-			 "website", "http://www.gnome.org/projects/gnome-games/",
-			 "wrap-license", TRUE,
-			 NULL);
+			 "comments",
+			 _("AisleRiot provides a rule-based solitaire\n"
+			   "card engine that allows many different\n"
+			   "games to be played."), "copyright",
+			 "Copyright \xc2\xa9 1998-2006 Jonathan Blandford",
+			 "license", license, "authors", authors,
+			 "documenters", documenters, "translator_credits",
+			 _("translator-credits"), "logo-icon-name",
+			 "gnome-aisleriot", "website",
+			 "http://www.gnome.org/projects/gnome-games/",
+			 "wrap-license", TRUE, NULL);
+  g_free (license);
 }
 
-static void general_help (void)
+static void
+general_help (void)
 {
   gnome_help_display_on_screen ("aisleriot.xml", NULL,
 				gtk_widget_get_screen (app), NULL);
 }
 
-static void help_on_specific_game (void)
+static void
+help_on_specific_game (void)
 {
   gchar *help_section;
   
   help_section = game_file_to_help_section (game_file);
   gnome_help_display_on_screen ("aisleriot.xml", help_section,
- 				gtk_widget_get_screen (app), NULL);
+				gtk_widget_get_screen (app), NULL);
   g_free (help_section);
 }
 
-static void toolbar_toggle_callback(GtkToggleAction * togglebutton, 
-				    gpointer data)
+static void
+toolbar_toggle_callback (GtkToggleAction * togglebutton, gpointer data)
 {
   gboolean state;
-  
+
   state = gtk_toggle_action_get_active (togglebutton);
 
   if (state) {
     gtk_widget_show (toolbar);
     gconf_client_set_bool (gconf_client, "/apps/aisleriot/show_toolbar", TRUE,
-                           NULL);
+			   NULL);
   } else {
     gtk_widget_hide (toolbar);
-    gconf_client_set_bool (gconf_client, "/apps/aisleriot/show_toolbar", FALSE,
-                           NULL);
+    gconf_client_set_bool (gconf_client, "/apps/aisleriot/show_toolbar",
+			   FALSE, NULL);
   }
 }
 
-static void set_fullscreen_actions (gboolean is_fullscreen)
+static void
+set_fullscreen_actions (gboolean is_fullscreen)
 {
   if (is_fullscreen) {
     gtk_widget_hide (menubar);
@@ -172,7 +197,7 @@ static void set_fullscreen_actions (gboolean is_fullscreen)
 }
 
 static void
-fullscreen_callback (GtkAction *action)
+fullscreen_callback (GtkAction * action)
 {
   if (action == fullscreenaction)
     gtk_window_fullscreen (GTK_WINDOW (app));
@@ -180,8 +205,8 @@ fullscreen_callback (GtkAction *action)
     gtk_window_unfullscreen (GTK_WINDOW (app));
 }
 
-static void window_state_callback (GtkWidget *widget, 
-				   GdkEventWindowState *event)
+static void
+window_state_callback (GtkWidget * widget, GdkEventWindowState * event)
 {
   if (!(event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN))
     return;
@@ -190,16 +215,17 @@ static void window_state_callback (GtkWidget *widget,
 			  GDK_WINDOW_STATE_FULLSCREEN);
 }
 
-static void clickmove_toggle_callback(GtkToggleAction * 
-				      togglebutton, gpointer data)
+static void
+clickmove_toggle_callback (GtkToggleAction * togglebutton, gpointer data)
 {
   click_to_move = gtk_toggle_action_get_active (togglebutton);
 
-  gconf_client_set_bool (gconf_client, "/apps/aisleriot/click_to_move", 
+  gconf_client_set_bool (gconf_client, "/apps/aisleriot/click_to_move",
 			 click_to_move, NULL);
 }
 
-void undo_set_sensitive (gboolean state)
+void
+undo_set_sensitive (gboolean state)
 {
   gtk_action_set_sensitive (undoaction, state);
 
@@ -207,120 +233,144 @@ void undo_set_sensitive (gboolean state)
   gtk_action_set_sensitive (restartaction, state);
 }
 
-void redo_set_sensitive (gboolean state)
+void
+redo_set_sensitive (gboolean state)
 {
   gtk_action_set_sensitive (redoaction, state);
 }
 
 static const GtkActionEntry actions[] = {
-  { "GameMenu", NULL, N_("_Game") },
-  { "ViewMenu", NULL, N_("_View") },
-  { "ControlMenu", NULL, N_("_Control") },
-  { "OptionsMenu", NULL, "Options" /* not translated on purpose */ },
-  { "HelpMenu", NULL, N_("_Help") },
+  {"GameMenu", NULL, N_("_Game")},
+  {"ViewMenu", NULL, N_("_View")},
+  {"ControlMenu", NULL, N_("_Control")},
+  {"OptionsMenu", NULL, "Options" /* not translated on purpose */ },
+  {"HelpMenu", NULL, N_("_Help")},
 
-  { "NewGame", GAMES_STOCK_NEW_GAME, NULL, NULL, N_("Start a new game"), G_CALLBACK (new_game_action) },
-  { "RestartGame", GAMES_STOCK_RESTART_GAME, NULL, NULL, N_("Restart the current game"), G_CALLBACK (restart_game) },
-  { "Select", GTK_STOCK_INDEX, N_("_Select Game..."), "<Ctrl>o", N_("Play a different game"), G_CALLBACK (show_select_game_dialog) },
-  { "Statistics", GTK_STOCK_ADD, N_("S_tatistics"), NULL, N_("Show gameplay statistics"), G_CALLBACK (show_statistics_dialog) },
-  { "Quit", GTK_STOCK_QUIT, NULL, NULL, NULL, G_CALLBACK (quit_app) },
-  { "Fullscreen", GAMES_STOCK_FULLSCREEN, NULL, NULL, NULL, G_CALLBACK (fullscreen_callback) },
-  { "LeaveFullscreen", GAMES_STOCK_LEAVE_FULLSCREEN, NULL, NULL, NULL, G_CALLBACK (fullscreen_callback) },
-  { "Cards", NULL, N_("_Cards..."), NULL, NULL, G_CALLBACK (show_preferences_dialog) },
+  {"NewGame", GAMES_STOCK_NEW_GAME, NULL, NULL, N_("Start a new game"),
+   G_CALLBACK (new_game_action)},
+  {"RestartGame", GAMES_STOCK_RESTART_GAME, NULL, NULL,
+   N_("Restart the current game"), G_CALLBACK (restart_game)},
+  {"Select", GTK_STOCK_INDEX, N_("_Select Game..."), "<Ctrl>o",
+   N_("Play a different game"), G_CALLBACK (show_select_game_dialog)},
+  {"RecentlyPlayed", NULL, N_("Recently _Played")},
+  {"Statistics", GTK_STOCK_ADD, N_("S_tatistics"), NULL,
+   N_("Show gameplay statistics"), G_CALLBACK (show_statistics_dialog)},
+  {"Quit", GTK_STOCK_QUIT, NULL, NULL, NULL, G_CALLBACK (quit_app)},
+  {"Fullscreen", GAMES_STOCK_FULLSCREEN, NULL, NULL, NULL,
+   G_CALLBACK (fullscreen_callback)},
+  {"LeaveFullscreen", GAMES_STOCK_LEAVE_FULLSCREEN, NULL, NULL, NULL,
+   G_CALLBACK (fullscreen_callback)},
+  {"Cards", NULL, N_("_Cards..."), NULL, NULL,
+   G_CALLBACK (show_preferences_dialog)},
 
-  { "UndoMove", GAMES_STOCK_UNDO_MOVE, NULL, NULL, N_("Undo the last move"), G_CALLBACK (undo_callback) },
-  { "RedoMove", GAMES_STOCK_REDO_MOVE, NULL, NULL, N_("Redo the last move"), G_CALLBACK (redo_callback) },
-  { "Hint", GAMES_STOCK_HINT, NULL, NULL, N_("Show a hint"), G_CALLBACK (show_hint_dialog) },
-  { "Contents", GAMES_STOCK_CONTENTS, NULL, NULL, N_("View help for Aisleriot"), G_CALLBACK (general_help) },
-  { "Help", GTK_STOCK_HELP, NULL, "<Shift>F1", N_("View help for this game"), G_CALLBACK (help_on_specific_game) },
-  { "About", GTK_STOCK_ABOUT, NULL, NULL, NULL, G_CALLBACK (help_about_callback) },
+  {"UndoMove", GAMES_STOCK_UNDO_MOVE, NULL, NULL, N_("Undo the last move"),
+   G_CALLBACK (undo_callback)},
+  {"RedoMove", GAMES_STOCK_REDO_MOVE, NULL, NULL, N_("Redo the last move"),
+   G_CALLBACK (redo_callback)},
+  {"Hint", GAMES_STOCK_HINT, NULL, NULL, N_("Show a hint"),
+   G_CALLBACK (show_hint_dialog)},
+  {"Contents", GAMES_STOCK_CONTENTS, NULL, NULL,
+   N_("View help for Aisleriot"), G_CALLBACK (general_help)},
+  {"Help", GTK_STOCK_HELP, NULL, "<Shift>F1", N_("View help for this game"),
+   G_CALLBACK (help_on_specific_game)},
+  {"About", GTK_STOCK_ABOUT, NULL, NULL, NULL,
+   G_CALLBACK (help_about_callback)},
 };
 
 static const GtkToggleActionEntry toggles[] = {
-  { "Toolbar", NULL, N_("_Toolbar"), NULL, "Show or hide the toolbar", G_CALLBACK (toolbar_toggle_callback) },
-  { "ClickToMove", NULL, N_("_Click to Move"), NULL, "Pick up and drop cards by clicking", G_CALLBACK (clickmove_toggle_callback) },
+  {"Toolbar", NULL, N_("_Toolbar"), NULL, "Show or hide the toolbar",
+   G_CALLBACK (toolbar_toggle_callback)},
+  {"ClickToMove", NULL, N_("_Click to Move"), NULL,
+   "Pick up and drop cards by clicking",
+   G_CALLBACK (clickmove_toggle_callback)},
 };
 
-static const char ui_description[] = 
-"<ui>"
-"  <menubar name='MainMenu'>"
-"    <menu action='GameMenu'>"
-"      <menuitem action='NewGame'/>"
-"      <menuitem action='RestartGame'/>"
-"      <menuitem action='Select'/>"
-"      <menuitem action='Statistics'/>"
-"      <separator/>"
-"      <menuitem action='Quit'/>"
-"    </menu>"
-"    <menu action='ViewMenu'>"
-"      <menuitem action='Fullscreen'/>"
-"      <menuitem action='LeaveFullscreen'/>"
-"      <menuitem action='Toolbar'/>"
-"      <separator/>"
-"      <menuitem action='Cards'/>"
-"    </menu>"
-"    <menu action='ControlMenu'>"
-"      <menuitem action='ClickToMove'/>"
-"      <separator/>"
-"      <menuitem action='UndoMove'/>"
-"      <menuitem action='RedoMove'/>"
-"      <menuitem action='Hint'/>"
-"    </menu>"
-"    <menu action='OptionsMenu'/>"
-"    <menu action='HelpMenu'>"
-"      <menuitem action='Contents'/>"
-"      <menuitem action='Help'/>"
-"      <menuitem action='About'/>"
-"    </menu>"
-"  </menubar>"
-"  <toolbar name='Toolbar'>"
-"    <toolitem action='NewGame'/>"
-"    <toolitem action='RestartGame'/>"
-"    <toolitem action='Select'/>"
-"    <separator/>"
-"    <toolitem action='UndoMove'/>"
-"    <toolitem action='RedoMove'/>"
-"    <toolitem action='Hint'/>" 
-"    <toolitem action='LeaveFullscreen'/>"
-"  </toolbar>"
-"</ui>";
+static const char ui_description[] =
+  "<ui>"
+  "  <menubar name='MainMenu'>"
+  "    <menu action='GameMenu'>"
+  "      <menuitem action='NewGame'/>"
+  "      <menuitem action='RestartGame'/>"
+  "      <menuitem action='Select'/>"
+  "      <menu action='RecentlyPlayed'/>"
+  "      <menuitem action='Statistics'/>"
+  "      <separator/>"
+  "      <menuitem action='Quit'/>"
+  "    </menu>"
+  "    <menu action='ViewMenu'>"
+  "      <menuitem action='Fullscreen'/>"
+  "      <menuitem action='LeaveFullscreen'/>"
+  "      <menuitem action='Toolbar'/>"
+  "      <separator/>"
+  "      <menuitem action='Cards'/>"
+  "    </menu>"
+  "    <menu action='ControlMenu'>"
+  "      <menuitem action='ClickToMove'/>"
+  "      <separator/>"
+  "      <menuitem action='UndoMove'/>"
+  "      <menuitem action='RedoMove'/>"
+  "      <menuitem action='Hint'/>"
+  "    </menu>"
+  "    <menu action='OptionsMenu'/>"
+  "    <menu action='HelpMenu'>"
+  "      <menuitem action='Contents'/>"
+  "      <menuitem action='Help'/>"
+  "      <menuitem action='About'/>"
+  "    </menu>"
+  "  </menubar>"
+  "  <toolbar name='Toolbar'>"
+  "    <toolitem action='NewGame'/>"
+  "    <toolitem action='RestartGame'/>"
+  "    <toolitem action='Select'/>"
+  "    <separator/>"
+  "    <toolitem action='UndoMove'/>"
+  "    <toolitem action='RedoMove'/>"
+  "    <toolitem action='Hint'/>"
+  "    <toolitem action='LeaveFullscreen'/>" "  </toolbar>" "</ui>";
 
-void create_menus ()
+void
+create_menus ()
 {
   GtkAccelGroup *accel_group;
   GtkAction *ctmtoggle;
   GtkAction *toolbartoggle;
 
-  games_stock_init();
+  games_stock_init ();
   action_group = gtk_action_group_new ("MenuActions");
   gtk_action_group_set_translation_domain (action_group, GETTEXT_PACKAGE);
+
   gtk_action_group_add_actions (action_group, actions, G_N_ELEMENTS (actions),
-				NULL); 
-  gtk_action_group_add_toggle_actions (action_group, toggles, 
+				NULL);
+  gtk_action_group_add_toggle_actions (action_group, toggles,
 				       G_N_ELEMENTS (toggles), NULL);
 
   ui_manager = gtk_ui_manager_new ();
   games_stock_prepare_for_statusbar_tooltips (ui_manager, statusbar);
 
-  gtk_ui_manager_insert_action_group (ui_manager, action_group, 1); 
+  gtk_ui_manager_insert_action_group (ui_manager, action_group, 1);
   gtk_ui_manager_add_ui_from_string (ui_manager, ui_description, -1, NULL);
 
   accel_group = gtk_ui_manager_get_accel_group (ui_manager);
   gtk_window_add_accel_group (GTK_WINDOW (app), accel_group);
 
+  /* The actions and menus are done. Add the recently played games */
+  install_recently_played_menu ();
+
   undoaction = gtk_action_group_get_action (action_group, "UndoMove");
   redoaction = gtk_action_group_get_action (action_group, "RedoMove");
   restartaction = gtk_action_group_get_action (action_group, "RestartGame");
   fullscreenaction = gtk_action_group_get_action (action_group, "Fullscreen");
-  leavefullscreenaction = gtk_action_group_get_action (action_group, "LeaveFullscreen");
+  leavefullscreenaction =
+    gtk_action_group_get_action (action_group, "LeaveFullscreen");
   ctmtoggle = gtk_action_group_get_action (action_group, "ClickToMove");
   toolbartoggle = gtk_action_group_get_action (action_group, "Toolbar");
 
-  helpitem = gtk_ui_manager_get_widget (ui_manager, "/MainMenu/HelpMenu/Help");
+  helpitem =
+    gtk_ui_manager_get_widget (ui_manager, "/MainMenu/HelpMenu/Help");
 
   menubar = gtk_ui_manager_get_widget (ui_manager, "/MainMenu");
   toolbar = gtk_ui_manager_get_widget (ui_manager, "/Toolbar");
-  
+
   gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (toolbartoggle),
 				gconf_client_get_bool (gconf_client,
 						       "/apps/aisleriot/show_toolbar",
@@ -335,18 +385,20 @@ void create_menus ()
   set_fullscreen_actions (FALSE);
 }
 
-void help_update_game_name ()
+void
+help_update_game_name ()
 {
   gtk_label_set_text (GTK_LABEL (GTK_BIN (helpitem)->child), game_name);
 }
 
-static gchar * make_option_gconf_key (void)
+static gchar *
+make_option_gconf_key (void)
 {
   static gchar *basekey = "/apps/aisleriot/rules/";
   gchar *r, *sk;
   GConfSchema *schema;
   GConfValue *def;
-    
+
   r = g_strconcat (basekey, game_file, NULL);
   sk = g_strconcat ("/schemas", r, NULL);
 
@@ -358,14 +410,15 @@ static gchar * make_option_gconf_key (void)
     gconf_schema_set_owner (schema, "aisleriot");
     /* FIXME: Translation - how? */
     gconf_schema_set_short_desc (schema, "A per-game option");
-    gconf_schema_set_long_desc (schema, "An integer encoding a list of boolean values (LSB = first item) for use as options in an solitaire game.");
+    gconf_schema_set_long_desc (schema,
+				"An integer encoding a list of boolean values (LSB = first item) for use as options in an solitaire game.");
     def = gconf_value_new (GCONF_VALUE_INT);
     gconf_value_set_int (def, 0);
     gconf_schema_set_default_value (schema, def);
     gconf_value_free (def);
     gconf_client_set_schema (gconf_client, sk, schema, NULL);
     gconf_engine_associate_schema (gconf_engine_get_default (), r, sk, NULL);
-  } 
+  }
   gconf_schema_free (schema);
   g_free (sk);
 
@@ -373,7 +426,8 @@ static gchar * make_option_gconf_key (void)
 }
 
 /* Make something that gconf can easily deal with. */
-static gint compress_options_to_int (SCM options_list)
+static gint
+compress_options_to_int (SCM options_list)
 {
   gint l, i;
   guint bits;
@@ -381,8 +435,8 @@ static gint compress_options_to_int (SCM options_list)
 
   bits = 0;
   l = SCM_INUM (scm_length (options_list)) - 1;
-  for (i=l; i>=0; i--) {
-    entry  = scm_list_ref (options_list, SCM_MAKINUM (i));
+  for (i = l; i >= 0; i--) {
+    entry = scm_list_ref (options_list, SCM_MAKINUM (i));
     if (SCM_NFALSEP (scm_list_p (entry))) {
       bits <<= 1;
       bits |= SCM_NFALSEP (scm_list_ref (entry, SCM_MAKINUM (1))) ? 1 : 0;
@@ -393,16 +447,18 @@ static gint compress_options_to_int (SCM options_list)
 }
 
 /* Take the gconf bit-string value and set the option list from it. */
-static void expand_options_from_int (SCM options_list, guint bits)
+static void
+expand_options_from_int (SCM options_list, guint bits)
 {
   gint l, i;
   SCM entry;
 
   l = SCM_INUM (scm_length (options_list));
-  for (i=0; i<l; i++) {
-    entry  = scm_list_ref (options_list, SCM_MAKINUM (i));
+  for (i = 0; i < l; i++) {
+    entry = scm_list_ref (options_list, SCM_MAKINUM (i));
     if (SCM_NFALSEP (scm_list_p (entry))) {
-      scm_list_set_x (entry, SCM_MAKINUM (1), bits & 1 ? SCM_BOOL_T : SCM_BOOL_F);
+      scm_list_set_x (entry, SCM_MAKINUM (1),
+		      bits & 1 ? SCM_BOOL_T : SCM_BOOL_F);
       bits >>= 1;
     }
   }
@@ -411,44 +467,46 @@ static void expand_options_from_int (SCM options_list, guint bits)
 /* FIXME: There isn't a schema for these automatically generated keys.
  * Maybe we should automatically generate those too. */
 
-void option_list_set_sensitive (void)
+void
+option_list_set_sensitive (void)
 {
   if (menuaction == NULL)
     return;
-  
+
   gtk_action_set_sensitive (menuaction, !game_in_progress);
 }
 
-static void save_option_list (SCM options_list)
+static void
+save_option_list (SCM options_list)
 {
   gchar *keyname;
 
   keyname = make_option_gconf_key ();
 
-  gconf_client_set_int (gconf_client, keyname, 
-			compress_options_to_int (options_list), 
-			NULL);
-			
+  gconf_client_set_int (gconf_client, keyname,
+			compress_options_to_int (options_list), NULL);
+
   g_free (keyname);
 }
 
-static void load_option_list (SCM options_list)
+static void
+load_option_list (SCM options_list)
 {
   gchar *keyname;
   guint r;
 
   keyname = make_option_gconf_key ();
 
-  r = gconf_client_get_int (gconf_client, keyname, 
-			    NULL);
-			
+  r = gconf_client_get_int (gconf_client, keyname, NULL);
+
   g_free (keyname);
 
   expand_options_from_int (options_list, r);
   cscmi_apply_options_lambda (options_list);
 }
 
-static void option_cb (GtkToggleAction *action, gint n)
+static void
+option_cb (GtkToggleAction * action, gint n)
 {
   SCM options_list;
   SCM entry;
@@ -460,7 +518,7 @@ static void option_cb (GtkToggleAction *action, gint n)
 
   statec = gtk_toggle_action_get_active (action);
 
-  scm_list_set_x (entry, SCM_MAKINUM(1), statec ? SCM_BOOL_T : SCM_BOOL_F);
+  scm_list_set_x (entry, SCM_MAKINUM (1), statec ? SCM_BOOL_T : SCM_BOOL_F);
 
   save_option_list (options_list);
   cscmi_apply_options_lambda (options_list);
@@ -479,7 +537,8 @@ enum {
  * another atom. The exact value of the atoms is irrelevant - they merely
  * trigger a toggle - but descriptive names like begin-exclusive and
  * end-exclusive are probably a good idea. */
-void install_options_menu ()
+void
+install_options_menu ()
 {
   static int merge_id = 0;
   static GtkActionGroup *options_group = NULL;
@@ -501,17 +560,18 @@ void install_options_menu ()
   if (cscmi_has_options ()) {
     menuaction = gtk_action_group_get_action (action_group, "OptionsMenu");
     g_object_set (G_OBJECT (menuaction), "label", game_name, NULL);
+
     options_group = gtk_action_group_new ("OptionsActions");
     gtk_ui_manager_insert_action_group (ui_manager, options_group, -1);
     g_object_unref (options_group);
 
     merge_id = gtk_ui_manager_new_merge_id (ui_manager);
-    
+
     options_list = cscmi_get_options_lambda ();
     load_option_list (options_list);
     l = SCM_INUM (scm_length (options_list));
     /* FIXME: We still leak a little here, especially actions. */
-    for (i=0; i<l; i++) {
+    for (i = 0; i < l; i++) {
       SCM entry;
       gchar *entryname;
       gboolean entrystate;
@@ -519,44 +579,114 @@ void install_options_menu ()
       gchar actionname[20];
 
       /* Each entry in the options list is a list consisting of a name and 
-	 a variable. */
+       * a variable. */
       entry = scm_list_ref (options_list, SCM_MAKINUM (i));
       if (SCM_NFALSEP (scm_list_p (entry))) {
 	entryname = SCM_STRING_CHARS (scm_list_ref (entry, SCM_MAKINUM (0)));
 	entrystate = SCM_NFALSEP (scm_list_ref (entry, SCM_MAKINUM (1)));
-	
+
 	g_snprintf (actionname, sizeof (actionname), "Option%d", i);
-	
-	if (mode == menu_normal) { 
-	  itemaction = gtk_toggle_action_new (actionname, entryname, 
+
+	if (mode == menu_normal) {
+	  itemaction = gtk_toggle_action_new (actionname, entryname,
 					      NULL, NULL);
 	} else {
-	  itemaction = GTK_TOGGLE_ACTION (gtk_radio_action_new (actionname, 
-							 entryname, 
-							 NULL, NULL, i));
-	  gtk_radio_action_set_group (GTK_RADIO_ACTION (itemaction), radiogroup);
-	  radiogroup = gtk_radio_action_get_group (GTK_RADIO_ACTION (itemaction));
+	  itemaction = GTK_TOGGLE_ACTION (gtk_radio_action_new (actionname,
+								entryname,
+								NULL, NULL,
+								i));
+	  gtk_radio_action_set_group (GTK_RADIO_ACTION (itemaction),
+				      radiogroup);
+	  radiogroup =
+	    gtk_radio_action_get_group (GTK_RADIO_ACTION (itemaction));
 	}
 	g_signal_connect (G_OBJECT (itemaction), "toggled",
-			  G_CALLBACK (option_cb), 
-			  GINT_TO_POINTER (i));
+			  G_CALLBACK (option_cb), GINT_TO_POINTER (i));
 	gtk_action_group_add_action (action_group, GTK_ACTION (itemaction));
 	gtk_toggle_action_set_active (itemaction, entrystate);
 	g_object_unref (itemaction);
-	
+
 	gtk_ui_manager_add_ui (ui_manager, merge_id, "/MainMenu/OptionsMenu",
 			       actionname, actionname,
 			       GTK_UI_MANAGER_MENUITEM, FALSE);
       } else {
 	/* If we encounter an atom, change the mode. What the atom is doesn't
-	   really matter. */
+	 * really matter. */
 	if (mode == menu_normal) {
 	  mode = menu_radio;
-	  radiogroup = NULL; /* Start a new radio group. */
+	  radiogroup = NULL;	/* Start a new radio group. */
 	} else {
 	  mode = menu_normal;
 	}
       }
     }
-  } 
+  }
+}
+
+/*
+ * Set up a sub-menu with the recently played games
+ * We'll keep track of 5 different games
+ */
+void
+install_recently_played_menu ()
+{
+  char *game_file_name = NULL;
+  gchar *game_name = NULL;
+  gchar *callback_data = NULL;
+
+  GtkAction *pAction = NULL;
+  GtkWidget *pWidget = NULL;
+
+  /* Show the empty menu if they don't have any recent games yet */
+  GtkAction *action =
+    gtk_action_group_get_action (action_group, "RecentlyPlayed");
+  if (action) {
+    g_object_set (action, "hide_if_empty", FALSE, NULL);
+  }
+
+  /* Get a list of the recent games */
+  GConfClient *gconf_client = gconf_client_get_default ();
+
+  gchar *recent_games =
+    gconf_client_get_string (gconf_client, RECENT_GAMES_GCONF_KEY, NULL);
+
+  if (recent_games == NULL)
+    return;
+
+  /* Get the MenuItem that the RecentlyPlayed menu belongs to */
+  GtkWidget *recent_menu =
+    gtk_ui_manager_get_widget (ui_manager,
+			       "/MainMenu/GameMenu/RecentlyPlayed");
+
+  if (recent_menu == NULL)
+    return;
+
+  gtk_menu_item_remove_submenu (GTK_MENU_ITEM (recent_menu));
+
+  GtkWidget *submenu = gtk_menu_new ();
+
+  game_file_name = strtok (recent_games, ",");
+
+  while (game_file_name != NULL) {
+    /* Create actions for the recent games */
+    game_name = game_file_to_name (game_file_name);
+    callback_data = g_strdup (game_file_name);
+
+    pAction = gtk_action_new (game_name, game_name, NULL, NULL);
+
+    g_signal_connect (G_OBJECT (pAction), "activate",
+		      G_CALLBACK (new_recent_game_action), callback_data);
+
+    /* Turn them into widgets */
+    pWidget = gtk_action_create_menu_item (pAction);
+
+    /* Append the recent games to the submenu */
+    gtk_menu_shell_append (GTK_MENU_SHELL (submenu), pWidget);
+
+    game_file_name = strtok (NULL, ",");
+  }
+
+  gtk_menu_item_set_submenu (GTK_MENU_ITEM (recent_menu), submenu);
+
+  g_free (recent_games);
 }
